{% set data = collection.serializer.dictize_row(record) if collection.serializer.ensure_dictized else record %}

<tr class="collection-table--row">
    {% block rows %}
        {% for column in collection.columns.names if column in collection.columns.visible -%}
            <td class="collection-table--cell{% if sortable %} collection-table--cell-sortable{% endif %}">
                {% block value scoped %}
                    <div data-collection-column="{{ column }}">
                        {{ data[column] }}
                    </div>
                {% endblock %}
            </td>
        {%- endfor %}
    {% endblock rows%}
</tr>
